(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['exports', 'react', 'moment', 'react-dom', 'melon-core/InputComponent', 'melon-core/classname/cxBuilder', 'melon-core/Validity', 'melon-core/util/syncPropsToState', 'melon/Icon', 'melon/Confirm', './timepicker/Panel', 'lodash/omit', './babelHelpers'], factory);
    } else if (typeof exports !== "undefined") {
        factory(exports, require('react'), require('moment'), require('react-dom'), require('melon-core/InputComponent'), require('melon-core/classname/cxBuilder'), require('melon-core/Validity'), require('melon-core/util/syncPropsToState'), require('melon/Icon'), require('melon/Confirm'), require('./timepicker/Panel'), require('lodash/omit'), require('./babelHelpers'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod.exports, global.react, global.moment, global.reactDom, global.InputComponent, global.cxBuilder, global.Validity, global.syncPropsToState, global.Icon, global.Confirm, global.Panel, global.omit, global.babelHelpers);
        global.TimePicker = mod.exports;
    }
})(this, function (exports, _react, _moment, _reactDom, _InputComponent2, _cxBuilder, _Validity, _syncPropsToState, _Icon, _Confirm, _Panel, _omit, babelHelpers) {
    'use strict';

    exports.__esModule = true;

    var _react2 = babelHelpers.interopRequireDefault(_react);

    var _moment2 = babelHelpers.interopRequireDefault(_moment);

    var _reactDom2 = babelHelpers.interopRequireDefault(_reactDom);

    var _InputComponent3 = babelHelpers.interopRequireDefault(_InputComponent2);

    var _Validity2 = babelHelpers.interopRequireDefault(_Validity);

    var _Icon2 = babelHelpers.interopRequireDefault(_Icon);

    var _Confirm2 = babelHelpers.interopRequireDefault(_Confirm);

    var _Panel2 = babelHelpers.interopRequireDefault(_Panel);

    var _omit2 = babelHelpers.interopRequireDefault(_omit);

    /**
     * @file melon/TimePicker
     * @author cxtom <cxtom2008@gmail.com>
     */

    var cx = (0, _cxBuilder.create)('TimePicker');

    var TimePicker = function (_InputComponent) {
        babelHelpers.inherits(TimePicker, _InputComponent);

        function TimePicker(props, context) {
            babelHelpers.classCallCheck(this, TimePicker);

            var _this = babelHelpers.possibleConstructorReturn(this, _InputComponent.call(this, props, context));

            var value = _this.state.value;

            _this.onLabelClick = _this.onLabelClick.bind(_this);
            _this.onConfirm = _this.onConfirm.bind(_this);
            _this.onCancel = _this.onCancel.bind(_this);
            _this.onTimeChange = _this.onTimeChange.bind(_this);

            _this.state = babelHelpers['extends']({}, _this.state, {

                // 缓存用户在 confirm 前的选中值
                time: value ? _this.parseValue(value) : undefined,

                // 是否打开选择窗
                open: false
            });

            return _this;
        }

        TimePicker.prototype.componentDidMount = function componentDidMount() {

            _InputComponent.prototype.componentDidMount.call(this);

            var container = this.container = document.createElement('div');

            container.className = cx().part('popup').build();

            document.body.appendChild(container);
            this.renderPopup(this.props);
        };

        TimePicker.prototype.componentDidUpdate = function componentDidUpdate() {
            this.renderPopup(this.props);
        };

        TimePicker.prototype.renderPopup = function renderPopup(props) {

            var timeFormat = props.timeFormat;

            var begin = props.begin,
                end = props.end;


            begin = begin ? this.parseValue(begin) : null;
            end = end ? this.parseValue(end) : null;

            this.popup = _reactDom2['default'].render(_react2['default'].createElement(
                _Confirm2['default'],
                {
                    open: this.state.open,
                    variants: ['timepicker'],
                    onConfirm: this.onConfirm,
                    onCancel: this.onCancel,
                    onShow: this.props.onFocus,
                    onHide: this.props.onBlur,
                    width: 'adaptive',
                    buttonVariants: ['secondery', 'timepicker'] },
                _react2['default'].createElement(_Panel2['default'], {
                    time: this.state.time,
                    begin: begin,
                    end: end,
                    format: timeFormat,
                    onChange: this.onTimeChange })
            ), this.container);
        };

        TimePicker.prototype.componentWillUnmount = function componentWillUnmount() {

            var container = this.container;

            if (container) {
                _reactDom2['default'].unmountComponentAtNode(container);
                container.parentElement.removeChild(container);
                this.container = container = null;
            }
        };

        TimePicker.prototype.getSyncUpdates = function getSyncUpdates(nextProps) {
            var disabled = nextProps.disabled,
                readOnly = nextProps.readOnly,
                customValidity = nextProps.customValidity,
                defaultValue = nextProps.defaultValue;


            var value = nextProps.value == null ? defaultValue : nextProps.value;

            // 如果有值，那么就试着解析一下；否则设置为 null
            var time = value ? this.parseValue(value) : null;

            // 如果 time 为 null 或者是 invalid time，那么就用上默认值；
            // 否则就用解析出来的这个值
            time = !time || isNaN(time.getTime()) ? new Date() : time;

            var vilidity = (0, _syncPropsToState.getNextValidity)(this, { value: value, disabled: disabled, customValidity: customValidity });

            return {
                time: time,
                vilidity: vilidity,
                value: disabled || readOnly || !value ? value : this.stringifyValue(time)
            };
        };

        TimePicker.prototype.stringifyValue = function stringifyValue(time) {

            if (typeof time === 'string') {
                return time;
            }

            var timeFormat = this.props.timeFormat;

            return (0, _moment2['default'])(time).format(timeFormat);
        };

        TimePicker.prototype.parseValue = function parseValue(date) {

            if (typeof date !== 'string') {
                return date;
            }

            return (0, _moment2['default'])(date, this.props.timeFormat).toDate();
        };

        TimePicker.prototype.onLabelClick = function onLabelClick(e) {
            var _props = this.props,
                disabled = _props.disabled,
                readOnly = _props.readOnly;


            if (disabled || readOnly) {
                return;
            }

            this.setState({ open: true });
        };

        TimePicker.prototype.onConfirm = function onConfirm() {
            var _this2 = this;

            var _state = this.state,
                value = _state.value,
                time = _state.time;


            value = this.parseValue(value);

            var valueMoment = (0, _moment2['default'])(value, this.timeFormat);
            var timeMoment = (0, _moment2['default'])(time);

            if (value !== '' && valueMoment.get('hour') === timeMoment.get('hour') && valueMoment.get('minute') === timeMoment.get('minute')) {
                this.setState({ open: false });
            }

            this.setState({ open: false }, function () {

                _InputComponent.prototype.onChange.call(_this2, {
                    type: 'change',
                    target: _this2,
                    value: _this2.stringifyValue(time)
                });
            });
        };

        TimePicker.prototype.onCancel = function onCancel() {
            this.setState({ open: false });
        };

        TimePicker.prototype.onTimeChange = function onTimeChange(_ref) {
            var time = _ref.time;

            this.setState({ time: time });
        };

        TimePicker.prototype.renderHiddenInput = function renderHiddenInput() {
            var _props2 = this.props,
                name = _props2.name,
                value = _props2.value;


            return name ? _react2['default'].createElement('input', {
                name: name,
                type: 'hidden',
                value: value }) : null;
        };

        TimePicker.prototype.render = function render() {
            var state = this.state,
                props = this.props;
            var disabled = props.disabled,
                readOnly = props.readOnly,
                placeholder = props.placeholder,
                timeFormat = props.timeFormat,
                labelFormat = props.labelFormat,
                others = babelHelpers.objectWithoutProperties(props, ['disabled', 'readOnly', 'placeholder', 'timeFormat', 'labelFormat']);
            var value = state.value,
                validity = state.validity;


            var open = state.open;
            var className = cx(props).addStates({ focus: open }).addStates(this.getStyleStates()).build();

            return _react2['default'].createElement(
                'div',
                babelHelpers['extends']({}, (0, _omit2['default'])(others, ['autoConfirm', 'name']), { className: className }),
                this.renderHiddenInput(),
                _react2['default'].createElement(
                    'label',
                    {
                        onClick: disabled || readOnly ? null : this.onLabelClick,
                        className: cx.getPartClassName('label') },
                    value ? (0, _moment2['default'])(value, timeFormat).format(labelFormat) : _react2['default'].createElement(
                        'span',
                        { className: cx.getPartClassName('label-placeholder') },
                        placeholder
                    ),
                    _react2['default'].createElement(_Icon2['default'], { icon: 'expand-more' })
                ),
                _react2['default'].createElement(_Validity2['default'], { validity: validity })
            );
        };

        return TimePicker;
    }(_InputComponent3['default']);

    exports['default'] = TimePicker;


    TimePicker.displayName = 'TimePicker';

    TimePicker.defaultProps = babelHelpers['extends']({}, _InputComponent3['default'].defaultProps, {
        timeFormat: 'HH:mm:ss',
        labelFormat: 'HH:mm',
        placeholder: '请选择',
        autoConfirm: false
    });

    TimePicker.propTypes = babelHelpers['extends']({}, _InputComponent3['default'].propTypes, {
        value: _react.PropTypes.string,
        autoConfirm: _react.PropTypes.bool,
        timeFormat: _react.PropTypes.string,
        labalFormat: _react.PropTypes.string,
        end: _react.PropTypes.oneOfType([_react.PropTypes.object, _react.PropTypes.string]),
        begin: _react.PropTypes.oneOfType([_react.PropTypes.object, _react.PropTypes.string])
    });

    TimePicker.childContextTypes = _InputComponent3['default'].childContextTypes;
    TimePicker.contextTypes = _InputComponent3['default'].contextTypes;
});
//# sourceMappingURL=TimePicker.js.map
